﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

namespace KrakJam {
    public class TimerCountdownAudio : NAudioSource {

        [SerializeField]
        private AudioClip _countdownClip;
        [SerializeField]
        private AudioClip _endGameClip;
        [SerializeField]
        private List<AudioClip> _voClips;
        [SerializeField]
        private AudioSource _musicSource;
        [SerializeField]
        private AudioSource _scoreSource;
        [SerializeField]
        private AudioSource _voSource;
        [SerializeField]
        private AudioClip _voGreenLost;
        [SerializeField]
        private AudioClip _voOrangeLost;
        [SerializeField]
        private AudioClip _voDraw;

        private bool _countingDown;
        private GameManager_v2 _gameManager;

        protected override void Awake() {
            base.Awake();
        }

        private void Start() {
            _gameManager = GameManager_v2.instance;
        }

        public void Score() {
            _scoreSource.Play();
        }

        public void OnStartGame(int seconds) {
            _countingDown = false;
            StartCoroutine(SimpleCountdown(seconds));
            _musicSource.Play();
        }

        private IEnumerator SimpleCountdown(int seconds) {
            for(int i = seconds; i > 0; i--) {
                _audioSource.PlayOneShot(_countdownClip);
                _voSource.PlayOneShot(_voClips[i - 1]);
                yield return new WaitForSecondsRealtime(1f);
            }
        }

        private void Update() {
            //Debug.Log(_gameManager.gameTime - _gameManager.elapsedTime + " " + _countingDown);
            if (_gameManager.gameTime - _gameManager.elapsedTime <= 6 && !_countingDown) {
                StartCoroutine(Countdown());
                _countingDown = true;
            }
        }

        public IEnumerator Countdown() {
            yield return new WaitForSeconds(_gameManager.gameTime - _gameManager.elapsedTime - 5f);
            for (int i = (int)(_gameManager.gameTime - _gameManager.elapsedTime); i > 0; i--) {
                _audioSource.PlayOneShot(_countdownClip);
                _voSource.PlayOneShot(_voClips[i]);
                yield return new WaitForSeconds(1f);
            }
            _audioSource.PlayOneShot(_endGameClip);
            if (GameManager_v2.instance.ScoreTeamA > GameManager_v2.instance.ScoreTeamB) {
                _voSource.PlayOneShot(_voGreenLost);
            } else if (GameManager_v2.instance.ScoreTeamA > GameManager_v2.instance.ScoreTeamB) {
                _voSource.PlayOneShot(_voOrangeLost);
            } else {
                _voSource.PlayOneShot(_voDraw);
            }
        }
    }
}
