﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

namespace KrakJam {
    public class GameManager : GenericSingleton<GameManager> {

        public WaveType waveType { get { return _waveType; } }
        public float nextWaveElapsedTime { get { return _nextWaveElapsedTime; } }
        public float elapsedTime { get { return _elapsedTime; } }
        public float gameTime { get { return _gameTime; } }
        public bool gameIsOn { get { return _gameIsOn; } }

        [SerializeField]
        private float _gameTime = 300f;
        [SerializeField]
        private float _waveTime = 30f;

        private DynamicObjectsManager _dynamicObjectsManager;
        private PlayerManager _playerManager;

        private float _elapsedTime;
        private float _nextWaveElapsedTime;
        private bool _gameIsOn;
        private WaveType _waveType = WaveType.Normal;
        private int _numberOfTypesOfWaves;

        [SerializeField] private GameMenu _gameMenu;


        protected override void Awake() {
            base.Awake();
            _elapsedTime = 0f;
            _nextWaveElapsedTime = 0f;
            _gameIsOn = false;
            _numberOfTypesOfWaves = System.Enum.GetValues(typeof(WaveType)).Length;
        }

        protected override void Start() {
            _dynamicObjectsManager = DynamicObjectsManager.instance;
            _playerManager = PlayerManager.instance;
        }


        public void StartGame()
        {
            _elapsedTime = 0f;
            _nextWaveElapsedTime = 0f;
            _gameIsOn = true;
            StartCoroutine(PlayingMatch());
        }


        //private void Update() {
            //if (_gameIsOn) {
            //    if((_elapsedTime += Time.deltaTime) > _gameTime){
            //        _gameIsOn = false;
            //    }

            //    _gameMenu.UpdateGameTimer(_gameTime - _elapsedTime);

            //    if ((_nextWaveElapsedTime += Time.deltaTime) > _waveTime) {
            //        int waveType;
            //        do { waveType = Random.Range(0, _numberOfTypesOfWaves); } while ((WaveType)waveType == _waveType);
            //        _waveType = (WaveType)waveType;
            //        _dynamicObjectsManager.NewWave(_waveType);
            //        _playerManager.NewWave(_waveType);
            //        _nextWaveElapsedTime -= _waveTime;
            //    }

            //    _gameMenu.UpdateWaveTime(_nextWaveElapsedTime);
            //}
        //}

        
        private IEnumerator PlayingMatch()
        {
            // Match loop
            while (_gameIsOn)
            { 
                if ((_elapsedTime += Time.deltaTime) > _gameTime)
                {
                    _gameIsOn = false;
                }

                _gameMenu.UpdateGameTimer(_gameTime - _elapsedTime);

                if ((_nextWaveElapsedTime += Time.deltaTime ) > _waveTime)
                {
                    int waveType;
                    do {
                        waveType = Random.Range(0, _numberOfTypesOfWaves);
                    } while ((WaveType)waveType == _waveType);

                    _waveType = (WaveType)waveType;
                    _dynamicObjectsManager.NewWave(_waveType);
                    _playerManager.NewWave(_waveType);
                    _nextWaveElapsedTime -= _waveTime;
                }

                //_gameMenu.UpdateWaveTime(_waveTime - _nextWaveElapsedTime);
                _gameMenu.UpdateWaveSlider(_nextWaveElapsedTime / _waveTime);

                yield return null;
            }

            _gameMenu.EndGameMenu();
        }
    }
}
