﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.Events;

namespace KrakJam
{
    public class GameManager_v2 : GenericSingleton<GameManager_v2> {

        public WaveType waveType { get { return _waveType; } }
        public float nextWaveElapsedTime { get { return _nextWaveElapsedTime; } }
        public float elapsedTime { get { return _elapsedTime; } }
        public float gameTime { get { return _gameTime; } }
        public bool gameIsOn { get { return _gameIsOn; } }
        public float waveTime { get { return _waveTime; } }
        public int ScoreTeamA { get { return _scoreTeamA; } }
        public int ScoreTeamB { get { return _scoreTeamB; } }
        

        [SerializeField]
        private float _gameTime = 300f;
        [SerializeField]
        private float _waveTime = 30f;

        [SerializeField]
        private TimerCountdownAudio _timerCountdown;
        [SerializeField]
        private WaveEffect _waveEffect;

        private DynamicObjectsManager _dynamicObjectsManager;
        private PlayerManager _playerManager;

        private float _elapsedTime;
        private float _nextWaveElapsedTime;
        private bool _gameIsOn;
        private WaveType _waveType = WaveType.Normal;
        private int _numberOfTypesOfWaves;
        private bool _startedTransition;

        [SerializeField] private GameMenu _gameMenu;

        private int _scoreTeamA;
        private int _scoreTeamB;

        public static UnityEvent onScore = new UnityEvent();


        protected override void Awake() {
            base.Awake();
            _elapsedTime = 0f;
            _nextWaveElapsedTime = 0f;
            _gameIsOn = false;
            _numberOfTypesOfWaves = System.Enum.GetValues(typeof(WaveType)).Length;
        }

        protected override void Start() {
            _dynamicObjectsManager = DynamicObjectsManager.instance;
            _playerManager = PlayerManager.instance;
        }

        public void PrepareToStart(int seconds) {
            _timerCountdown.OnStartGame(seconds);
        }

        public void StartGame()
        {
            _elapsedTime = 0f;
            _nextWaveElapsedTime = 0f;
            _gameIsOn = true;
            _scoreTeamA = 0;
            _scoreTeamB = 0;
            StartCoroutine(PlayingMatch());
        }

        private void Update()
        {
            VibrationManager.Update();
        }

        public void UpdateWaveName(string name) {
            _gameMenu.UpdateWaveName(name);
        }

        //private void Update() {
        //if (_gameIsOn) {
        //    if((_elapsedTime += Time.deltaTime) > _gameTime){
        //        _gameIsOn = false;
        //    }

        //    _gameMenu.UpdateGameTimer(_gameTime - _elapsedTime);

        //    if ((_nextWaveElapsedTime += Time.deltaTime) > _waveTime) {
        //        int waveType;
        //        do { waveType = Random.Range(0, _numberOfTypesOfWaves); } while ((WaveType)waveType == _waveType);
        //        _waveType = (WaveType)waveType;
        //        _dynamicObjectsManager.NewWave(_waveType);
        //        _playerManager.NewWave(_waveType);
        //        _nextWaveElapsedTime -= _waveTime;
        //    }

        //    _gameMenu.UpdateWaveTime(_nextWaveElapsedTime);
        //}
        //}

        public string MatchWinner()
        {
            if (_scoreTeamA > _scoreTeamB)
            {
                return "The winner is Team A";
            }
            else if (_scoreTeamB > _scoreTeamA)
            {
                return "The winner is Team B";
            }
            return "There is no winner today!";
        }


        public int Winner()
        {
            if (_scoreTeamA > _scoreTeamB)
            {
                return 0;
            }
            else if (_scoreTeamB > _scoreTeamA)
            {
                return 1;
            }
            return 2;
        }


        public void Score(int team)
        {
            if (team == 0)
            {
                _scoreTeamA += 1;
                _gameMenu.UpdateScore(_scoreTeamA, true);
            }
            else
            {
                _scoreTeamB += 1;
                _gameMenu.UpdateScore(_scoreTeamB, false);
            }
            onScore.Invoke();
            _timerCountdown.Score();
        }


        private IEnumerator PlayingMatch()
        {
            // Match loop
            while (_gameIsOn)
            {
                if (( _elapsedTime += Time.deltaTime ) > _gameTime)
                {
                    _gameIsOn = false;
                }

                _gameMenu.UpdateGameTimer(_gameTime - _elapsedTime);

                Debug.Log(_nextWaveElapsedTime + " " + (_waveTime - 0.5f));
                if (!_startedTransition) {
                    if (_nextWaveElapsedTime >= (_waveTime - 0.5f)) {
                        _waveEffect.StartCoroutine(_waveEffect.StartWave());
                        _startedTransition = true;
                    }
                }

                if (( _nextWaveElapsedTime += Time.deltaTime ) > _waveTime)
                {
                    //int waveType;
                    //do
                    //{
                    //    waveType = Random.Range(0, _numberOfTypesOfWaves);
                    //} while ((WaveType)waveType == _waveType);

                    //_waveType = (WaveType)waveType;
                    //_dynamicObjectsManager.NewWave(_waveType);
                    //_playerManager.NewWave(_waveType);

                    WaveMods.instance.ChangeWave();
                    _startedTransition = false;
                    _nextWaveElapsedTime -= _waveTime;
                }

                //_gameMenu.UpdateWaveTime(_waveTime - _nextWaveElapsedTime);
                _gameMenu.UpdateWaveSlider(_nextWaveElapsedTime / _waveTime);

                yield return null;
            }

            _gameMenu.EndGameMenu();
            
        }
    }
}
