﻿using UnityEngine;
using UnityEngine.UI;
using System.Collections;
using XInputDotNetPure;
using UnityEngine.SceneManagement;


namespace KrakJam
{
    public class GameMenu : MonoBehaviour
    {
        #region Fields and properties

        public bool modeSelected { get; set; }
        public int mode { get; set; }

        [Header("Canvas")]
        [SerializeField] private GameObject _startMenu;
        [SerializeField] private GameObject _hud;
        [SerializeField] private GameObject _endMenu;

        [Header("UI")]
        //[SerializeField] private Image[] _playerImages;
        [SerializeField] private Text _startTimerText;
        [SerializeField] private Text _gameTimerText;
        [SerializeField] private TextMesh _gameTimerTextMesh;
        [SerializeField] private Text _waveText;
        [SerializeField] private Text _waveTimer;
        [SerializeField] private Slider _waveSlider;
        [SerializeField] private Text _scoreTeamA;
        [SerializeField] private Text _scoreTeamB;
        //[SerializeField] private Text _gameOverText;
        //[SerializeField] private Text _scoreEnd_A;
        //[SerializeField] private Text _scoreEnd_B;

        [Header("Start UI")]
        [SerializeField] private GameObject _startCanvas;
        [SerializeField] private GameObject _backgroundImage;
        [SerializeField] private Image _title;
        [SerializeField] private Animator _titleAnimator;
        [SerializeField] private GameObject _selectModeView;
        [SerializeField] private GameObject _modeToSelectOnInit;
        [SerializeField] private GameObject _playerSelectionView;
        [SerializeField] private PlayerImage[] _pi;
        [SerializeField] private GameObject _counterView;
        [SerializeField] private Image _counter;
        [SerializeField] private Sprite[] _counterImages;
        [SerializeField] private GameObject _resetButton;

        [SerializeField] private Image _resultsImage;
        [SerializeField] private Sprite[] _resultsSprites;

        [Header("Data")]
        [SerializeField] private GameManager_v2 _gameManager;

        [SerializeField] private int _nPlayer = 2;  // game mode 1v1, 2v2
        [SerializeField] private float _timeToStart = 1f;
        private int _totalPlayers = 4;
        private bool[] _isReady;

        [SerializeField] private bool _instaStart;


        #endregion


        #region Constructors and Initialization

        private void Awake()
        {
            SceneManager.LoadScene(1, LoadSceneMode.Additive);
        }


        private IEnumerator Start()
        {
            if (!_instaStart)
            {
                Time.timeScale = 0f;

                _backgroundImage.SetActive(true);
                _startCanvas.gameObject.SetActive(true);
                _title.gameObject.SetActive(true);
                
                //_startMenu.SetActive(true);

                GamePadState[] state = new GamePadState[_totalPlayers];
                GamePadState[] prevS = new GamePadState[_totalPlayers];

                _isReady = new bool[_totalPlayers];

                // for prev in frame 1
                for (int i = 0; i < _totalPlayers; ++i)
                {
                    state[i] = GamePad.GetState((PlayerIndex)i);
                }
                
                while (true)
                {
                    for (int i = 0; i < _totalPlayers; ++i)
                    {
                        prevS[i] = state[i];
                        state[i] = GamePad.GetState((PlayerIndex)i);
                    }

                    bool start = false;
                    for (int i = 0; i < _totalPlayers; ++i)
                    {
                        if (state[i].IsConnected && 
                            (state[i].Buttons.Start == ButtonState.Pressed))
                        {
                            start = true;
                        }
                    }

                    if (start)
                    {
                        //_titleAnimator.SetTrigger("Pressed");
                        //yield return new WaitForSeconds(0.3f);
                        break;
                    }
                    else
                    {
                        yield return null;
                    }
                }

                _title.gameObject.SetActive(false);
                _selectModeView.SetActive(true);

                UnityEngine.EventSystems.EventSystem.current.SetSelectedGameObject(_modeToSelectOnInit);

                yield return new WaitUntil(() => modeSelected);
                yield return null;

                _selectModeView.SetActive(false);
                _backgroundImage.SetActive(false);
                _playerSelectionView.SetActive(true);

                for (int i = 0; i < _totalPlayers; ++i)
                {
                    prevS[i] = state[i];
                    state[i] = GamePad.GetState((PlayerIndex)i);
                }

                int playerRaedy;
                while(true)
                {
                    playerRaedy = 0;
                    for (int i = 0; i < _totalPlayers; ++i)
                    {
                        prevS[i] = state[i];
                        state[i] = GamePad.GetState((PlayerIndex)i);


                        if (state[i].IsConnected)
                        {
                            _pi[i].gameObject.SetActive(true);

                            if (state[i].Buttons.A == ButtonState.Pressed &&
                                prevS[i].Buttons.A == ButtonState.Released)
                            {
                                _isReady[i] = !_isReady[i];
                                _pi[i].Select(_isReady[i]);
                            }

                            if (_isReady[i])
                            {
                                playerRaedy += 1;
                            }
                        }
                        else
                        {
                            _pi[i].gameObject.SetActive(false);
                        }
                    }


                    if (playerRaedy == mode)
                    //if (playerRaedy == 1)
                    {
                        break;
                    }
                    yield return null;
                }
                _playerSelectionView.SetActive(false);

                _gameManager.PrepareToStart(6);
                _counterView.SetActive(true);
                float timer = _timeToStart;
                while (timer > 0)
                {
                    if (timer > 5)
                    {
                        _counter.sprite = _counterImages[5];
                    }
                    else if (timer > 4)
                    {
                        _counter.sprite = _counterImages[4];
                    }
                    else if (timer > 3)
                    {
                        _counter.sprite = _counterImages[3];
                    }
                    else if (timer > 2)
                    {
                        _counter.sprite = _counterImages[2];
                    }
                    else if (timer > 1)
                    {
                        _counter.sprite = _counterImages[1];
                    }
                    else if (timer > 0)
                    {
                        _counter.sprite = _counterImages[0];
                    }
                    timer -= Time.unscaledDeltaTime;
                    yield return null;
                }
                _counterView.SetActive(false);
            }

            Time.timeScale = 1f;

            //PlayerSpawner.instance.CreatePlayers(mode);
            PlayerSpawner.instance.CreatePlayers(_isReady, mode);

            _gameManager.StartGame();
        }


        #endregion
        
        
        #region MonoBehaviour methods
        #endregion
        
        
        #region Public methods

        public void UpdateGameTimer(float timeLeft)
        {
            if (timeLeft > 0)
            {
                //_gameTimerText.text = /*"Time left: " +*/ timeLeft.ToString("0.0");
                if (timeLeft < 10f)
                {
                    _gameTimerTextMesh.text = timeLeft.ToString("0.00");
                }
                else if (timeLeft < 100f)
                {
                    _gameTimerTextMesh.text = timeLeft.ToString("0.0");
                }
                else
                {
                    _gameTimerTextMesh.text = timeLeft.ToString("0");
                }
                
            }
            else
            {
                //_gameTimerText.text = string.Empty;
                _gameTimerTextMesh.text = string.Empty;
            }
        }


        public void UpdateWaveTime(float timeLeft)
        {
            if (timeLeft > 0)
            {
                _waveTimer.text = "Time left: " + timeLeft.ToString("0.0");
            }
            else
            {
                _waveTimer.text = string.Empty;
            }
        }


        public void UpdateWaveSlider(float value)
        {
            _waveSlider.value = value;
        }


        public void EndGameMenu()
        {
            _hud.SetActive(false);
            _endMenu.SetActive(true);
            //_scoreEnd_A.text = _gameManager.ScoreTeamA.ToString();
            //_scoreEnd_B.text = _gameManager.ScoreTeamB.ToString();
            //_gameOverText.text = _gameManager.MatchWinner();
            UnityEngine.EventSystems.EventSystem.current.SetSelectedGameObject(_resetButton);
            _resultsImage.sprite = _resultsSprites[_gameManager.Winner()];
        }


        public void ResetLevel()
        {
            SceneManager.LoadScene(0);
        }


        public void UpdateScore(int score, bool teamA)
        {
            if (teamA)
            {
                _scoreTeamA.text = score.ToString();
            }
            else
            {
                _scoreTeamB.text = score.ToString();
            }
        }

        public void UpdateWaveName(string name) {
            _waveText.text = name;
        }


        public void Quit()
        {
#if UNITY_EDITOR
            UnityEditor.EditorApplication.isPlaying = false;
#else
            Application.Quit();
#endif
        }

        #endregion


        #region Protected methods
        #endregion


        #region Private methods
        #endregion

    } //class

} //namespace
