﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

namespace KrakJam {
    public class GenericSingleton<T> : MonoBehaviour where T : Component {

        public static T instance
        {
            get
            {
                if (!_instance) {
                    if (!(_instance = FindObjectOfType<T>())) {
                        _instance = new GameObject(typeof(T).ToString()).AddComponent<T>();
                    }
                }
                return _instance;
            }
        }

        private static T _instance;

        protected virtual void Awake() {
            if (_instance == null) {
                _instance = this as T;
            } else if (_instance != this) {
                Destroy(gameObject);
            }
        }

        protected virtual void Start() {

        }
    }
}
