﻿using UnityEngine;
using UnityEngine.Events;
using System.Collections;

namespace KrakJam
{
    public class WaveMods : GenericSingleton<WaveMods>
    {
        #region Fields and properties

        public WaveBase CurrentWave { get { return _currentWave; } }

        [SerializeField] private WaveBase _currentWave;
        [SerializeField] private WaveBase[] _waves;

        [SerializeField]
        private Colorful.ChannelMixer _channelMixer;
        [SerializeField]
        private Colorful.Frost _frost;
        [SerializeField]
        private UnityStandardAssets.ImageEffects.Fisheye _fisheye;
        [SerializeField]
        private Colorful.Wiggle _wiggle;
        [SerializeField]
        private Colorful.HueSaturationValue _hsv;
        [SerializeField]
        private UnityStandardAssets.ImageEffects.ColorCorrectionCurves _colorCorr;
        [SerializeField]
        private Lava _lava;

        [SerializeField]
        private AudioSource _announcementSource;

        // static event
        public static UnityEvent onWaveChange = new UnityEvent();

        #endregion


        #region Constructors and Initialization
        protected override void Start() {
            Camera cam = Camera.main;
            _channelMixer = cam.GetComponent<Colorful.ChannelMixer>();
            _frost = cam.GetComponent<Colorful.Frost>();

            _wiggle = cam.GetComponent<Colorful.Wiggle>();
            _fisheye = cam.GetComponent<UnityStandardAssets.ImageEffects.Fisheye>();
            _hsv = cam.GetComponent<Colorful.HueSaturationValue>();
            _colorCorr = cam.GetComponent<UnityStandardAssets.ImageEffects.ColorCorrectionCurves>();
            _lava = FindObjectOfType<Lava>();
        }
        #endregion


        #region MonoBehaviour methods
        #endregion


        #region Public methods

        public void ChangeWave()
        {
            WaveBase newWave = null;
            do
            {
                newWave = _waves[Random.Range(0, _waves.Length)];
            }
            while (newWave == _currentWave);
            _currentWave = newWave;
            Debug.Log("New wave: " + _currentWave.name);
            onWaveChange.Invoke();

            _channelMixer.enabled = _currentWave.areObjectsSliding;
            _frost.enabled = _currentWave.areObjectsSliding;

            _wiggle.enabled = _hsv.enabled = _fisheye.enabled = _currentWave.playerGravity == 0;
            _colorCorr.enabled = _currentWave.isLava;
            if (_currentWave.isLava) {
                _lava.StartCoroutine(_lava.SetLava());
                //StartCoroutine(SetLava());
            }

            GameManager_v2.instance.UpdateWaveName(_currentWave.waveName);
            if (_currentWave.announcement) {
                _announcementSource.PlayOneShot(_currentWave.announcement);
            }
        }

        #endregion


        #region Protected methods


        #endregion


        #region Private methods
        private IEnumerator MoveWave() {
            yield return null;
        }

        private IEnumerator SetLava() {
            float time = 0;
            while(time < 1) {
                time += Time.deltaTime;
                yield return null;
            }
            time = 1f;
            yield return new WaitForSeconds(GameManager_v2.instance.waveTime - 2);
            while (time > 0) {
                time -= Time.deltaTime;
                yield return null;
            }
            time = 0f;
        }
        #endregion

    } //class

} //namespace
