﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

namespace KrakJam {

    public enum WaveType { Normal, NoGravity, Frozen }

    public class DynamicObjectsManager : GenericSingleton<DynamicObjectsManager> {

        public PhysicsMaterial2D slidingMaterial { get { return _slidingMaterial; } }

        [SerializeField]
        private PhysicsMaterial2D _slidingMaterial;

        private List<Pickupable> _listOfDynamicObjects;

        protected override void Start() {
            _listOfDynamicObjects = new List<Pickupable>(FindObjectsOfType<Pickupable>());
        }

        public void NewWave(WaveType waveType) {
            switch (waveType) {
                case WaveType.NoGravity:
                    for (int i = 0; i < _listOfDynamicObjects.Count; i++) {
                        _listOfDynamicObjects[i].SetGravity(0f);
                    }
                    break;
                case WaveType.Normal:
                    for (int i = 0; i < _listOfDynamicObjects.Count; i++) {
                        _listOfDynamicObjects[i].SetGravity();
                        _listOfDynamicObjects[i].SetFriction(null);
                    }
                    break;
                case WaveType.Frozen:
                    for (int i = 0; i < _listOfDynamicObjects.Count; i++) {
                        _listOfDynamicObjects[i].SetGravity();
                        _listOfDynamicObjects[i].SetFriction(_slidingMaterial);
                    }
                    break;
            }
        }
    }
}
