﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

namespace KrakJam {
    public class Pickupable : MonoBehaviour {

        public PlayerController owner { get { return _owner; } }
        public new Rigidbody2D rigidbody { get { return _rigidbody; } }

        protected Rigidbody2D _rigidbody;
        private PlayerController _owner;
        protected Collider2D _collider;
        private float _gravityScale;
        private int _pickupableLayer;
        private int _pickedUpLayer;

        private Vector3 _startPos;

        protected virtual void Awake() {
            _rigidbody = GetComponent<Rigidbody2D>();
            _gravityScale = _rigidbody.gravityScale;
            _collider = GetComponent<Collider2D>();
            _pickedUpLayer = LayerMask.NameToLayer("PickedUp");
            _pickupableLayer = LayerMask.NameToLayer("Pickupable");
            _startPos = transform.position;
        }

        public void ResetPosition() {
            transform.position = _startPos;
            _rigidbody.velocity = Vector2.zero;
        }

        private void OnEnable()
        {
            WaveMods.onWaveChange.AddListener(SetWaveMods);
        }


        private void OnDisable()
        {
            WaveMods.onWaveChange.RemoveListener(SetWaveMods);
        }


        void FixedUpdate() {
            if (_owner) {
                _rigidbody.MovePosition(_owner.target.transform.position);
                //transform.position = _owner.target.transform.position;
            }
        }

        public void SetOwner(PlayerController playerController) {
            if (playerController) {
                _rigidbody.gravityScale = 0f;
                gameObject.layer = _pickedUpLayer;
            } else {
                _rigidbody.gravityScale = _gravityScale;
                if (_owner != null)
                {
                    _rigidbody.AddForce(( transform.position - _owner.transform.position ).normalized * 5000f);
                }
                else
                {
                    _rigidbody.velocity = Vector2.zero;
                }
                gameObject.layer = _pickupableLayer;
            }
            _owner = playerController;
        }

        public void SetGravity(float gravityScale = 1) {
            _gravityScale = gravityScale;
            _rigidbody.gravityScale = gravityScale;
        }

        public void SetFriction(PhysicsMaterial2D material) {
            _rigidbody.sharedMaterial = material;
        }

        private void SetWaveMods()
        {
            _gravityScale = WaveMods.instance.CurrentWave.dynamicObjectsGravity;
            _rigidbody.gravityScale = WaveMods.instance.CurrentWave.dynamicObjectsGravity;
            _rigidbody.sharedMaterial = WaveMods.instance.CurrentWave.areObjectsSliding ? DynamicObjectsManager.instance.slidingMaterial : null;
        }
    }
}