﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using XInputDotNetPure;
using Spine.Unity;

namespace KrakJam
{
    public enum JumpState { Grounded, Jump, InAir, DoubleJump, Falling }

    public class PlayerController : MonoBehaviour
    {
        #region Fields and properties
        public Transform SpawnPoint { set { _spawnPoint = value; } }
        public Transform SpawnPointHigh { set { _spawnPointHigh = value; } }
        public GameObject target { get { return _target; } }
        public List<Pickupable> dynamicObjectsInRange { get { return _dynamicObjectsInRange; } }

        [SerializeField] private PlayerIndex playerIndex;
        [SerializeField] private float speed;
        [SerializeField] private float _jumpPower = 2f;
        [SerializeField] private GameObject _target;
        [SerializeField] private Transform _spawnPoint;
        [SerializeField] private Transform _spawnPointHigh;
        [SerializeField] private PlayerAudioProxy _animatorAudioProxy;
        [SerializeField] private SkeletonAnimator _skeletonAnimator;
        [SerializeField] private float _respawnDelay = 2f;
        [SerializeField] private GameObject[] _bloodMarks;

        private Pickupable _pickedObject;

        private GamePadState state;
        private GamePadState prevState;

        [SerializeField] private Vector2 moveDirection;
        private Vector3 aimDirection;

        private Rigidbody2D _rigidbody;

        private bool _canJump = false;
        private bool _jumped = false;

        private bool _isGrounded = true;
        private bool _wasGrounded = false;

        private bool _shoot = false;
        private bool _shootReleased = true;

        private bool _isDead = false;

        private bool _facingRight = true;
        private int _jumpCount = 0;

        private bool _alternateControls = false;

        [SerializeField] private Animator _animator;
        private int idleAnim = Animator.StringToHash("idle");
        private int runAnim  = Animator.StringToHash("running");
        private int jumpAnim = Animator.StringToHash("jumping");
        private int die1Anim = Animator.StringToHash("die_1");
        private int die2Anim = Animator.StringToHash("die_2");

        [SpineSkin]
        public string skins1;
        [SpineSkin]
        public string skins2;
        [SpineSkin]
        public string skins3;
        [SpineSkin]
        public string skins4;

        private List<Pickupable> _dynamicObjectsInRange = new List<Pickupable>();
        #endregion


        #region Constructors and Initialization

        private void Awake()
        {
            _rigidbody = GetComponent<Rigidbody2D>();
        }


        public void Initialize(PlayerIndex index, int team)
        {
            playerIndex = index;
            FlipX(team == 0);       // if team 0 turn right, if team 1 turn left
            tag = ( team == 0 ) ? "TeamA" : "TeamB";
            int i = (int)index;
            switch (i)
            {
                case 0:
                    _skeletonAnimator.skeleton.SetSkin(skins1);
                    break;

                case 1:
                    _skeletonAnimator.skeleton.SetSkin(skins2);
                    break;

                case 2:
                    _skeletonAnimator.skeleton.SetSkin(skins3);
                    break;

                case 3:
                    _skeletonAnimator.skeleton.SetSkin(skins4);
                    break;
            }
        }
    


        private void OnEnable()
        {
            WaveMods.onWaveChange.AddListener(SetWaveMods);
            SetWaveMods();
            //GameManager_v2.onScore.AddListener(RespawnOnGoal);
        }


        private void OnDisable()
        {
            WaveMods.onWaveChange.RemoveListener(SetWaveMods);
            //GameManager_v2.onScore.RemoveListener(RespawnOnGoal);
        }

        #endregion


        #region MonoBehaviour methods

        public void Update() {
            // update states
            //Debug.Log(_rigidbody.velocity);


            if (Input.GetKeyDown(KeyCode.F4))
            {
                Die();
                return;
            }
            if (Input.GetKeyDown(KeyCode.F5)) {
                _alternateControls = !_alternateControls;
                Debug.Log("Alternate controls = " + _alternateControls);
            }

            if (_isDead || !GameManager_v2.instance.gameIsOn)
            {
                return;
            }

            prevState = state;
            state = GamePad.GetState(playerIndex);
            moveDirection = Vector2.zero;

            if (_rigidbody.gravityScale != 0) {
                if (_jumpCount < 2) {
                    if (!_jumped) {
                        bool jumped;
                        if (_alternateControls) {
                            jumped = state.Buttons.A == ButtonState.Pressed;
                        } else {
                            jumped = state.Buttons.LeftShoulder == ButtonState.Pressed;
                        }
                        if (jumped) {
                            _rigidbody.velocity = new Vector2(_rigidbody.velocity.x, 0f);
                            _animatorAudioProxy.PlayJump();
                            _rigidbody.AddForce(100f * _jumpPower * Vector2.up);
                            _jumped = true;
                            _jumpCount++;
                        }
                    } else {
                        bool released;
                        if (_alternateControls) {
                            released = state.Buttons.A == ButtonState.Released;
                        } else {
                            released = state.Buttons.LeftShoulder == ButtonState.Released;
                        }
                        if (released) {
                            _jumped = false;
                        }
                    }
                }
            }

            // movedirection
                if (state.ThumbSticks.Left.X > 0.1f) {
                    moveDirection.x = 1.0f;
                } else if (state.ThumbSticks.Left.X < -0.1f) {
                    moveDirection.x = -1.0f;
                } else {
                    moveDirection.x = 0f;
                }
            if(_rigidbody.gravityScale == 0) {
                if (state.ThumbSticks.Left.Y > 0.1f) {
                    moveDirection.y = 1.0f;
                } else if (state.ThumbSticks.Left.Y < -0.1f) {
                    moveDirection.y = -1.0f;
                } else {
                    moveDirection.y = 0f;
                }
            }

            // aim direction
            if (_alternateControls) {
                if (state.ThumbSticks.Left.X != 0) {
                    aimDirection.x = state.ThumbSticks.Left.X;
                }
                if (state.ThumbSticks.Left.Y != 0) {
                    aimDirection.y = state.ThumbSticks.Left.Y;
                }
            } else {
                if (state.ThumbSticks.Right.X != 0) {
                    aimDirection.x = state.ThumbSticks.Right.X;
                }
                if (state.ThumbSticks.Right.Y != 0) {
                    aimDirection.y = state.ThumbSticks.Right.Y;
                }
            }
            aimDirection.Normalize();

            _target.transform.localPosition = aimDirection * 1.7f;
            Vector3 vector = (target.transform.position - transform.position);
            Vector2 vec = Quaternion.Euler(0, 0, 45) * new Vector2(vector.x, vector.y);
            Debug.DrawLine(transform.position, transform.position + (Vector3)vec);

            bool weaponShot;
            if (_alternateControls) {
                weaponShot = state.Buttons.LeftShoulder == ButtonState.Pressed;
            } else {
                weaponShot = state.Buttons.RightShoulder == ButtonState.Pressed;
            }
            if (weaponShot) {
                if (!_shoot) {
                    _shoot = true;
                    if (_pickedObject) {
                        Debug.Log("tur");
                        _pickedObject.SetOwner(null);
                        _pickedObject = null;
                        _animatorAudioProxy.PlayGunPush();
                    } else {
                        for (int i = 0; i < _dynamicObjectsInRange.Count; i++) {
                            if (CheckDynamicObjectInRange(_dynamicObjectsInRange[i])) {
                                _animatorAudioProxy.PlayGunPull();
                                break;
                            }
                        }
                        _animatorAudioProxy.PlayGunMissing();
                    }
                }
            } else {
                if (_shoot) {
                    _shoot = false;
                }
            }
            //_shoot = (state.Triggers.Right != 0);


            // Update animations

            //if (_rigidbody.gravityScale == 0)
            //{
            //    // no gravity move
            //    _animator.SetBool(jumpAnim, true);
            //}
            //else 
            if (!_isGrounded)
            {
                // jump anim
                _animator.SetBool(jumpAnim, true);
            }
            else
            {
                _animator.SetBool(jumpAnim, false);
                if (moveDirection.x != 0)
                {
                    // run anim
                    _animator.SetBool(runAnim, true);
                }
                else
                {
                    // idle
                    _animator.SetBool(runAnim, false);
                }
            }

            if (_facingRight && ( aimDirection.x < 0 ))
            {
                FlipX(false);                    
            }
            else if (!_facingRight && ( aimDirection.x > 0))
            {
                FlipX(true);
            }

            if ( (aimDirection.x > 0 && moveDirection.x < 0) ||
                ( aimDirection.x < 0 && moveDirection.x > 0 ))
            { 
                _animator.SetBool("reverse", true);
            }
            else
            {
                _animator.SetBool("reverse", false);
            }
        }

        #endregion


        #region Public methods
        public void Die() {
            //transform.position = _spawnPoint.position;
            if (_pickedObject) {
                _pickedObject.SetOwner(null);
            }
            _animatorAudioProxy.PlayVoiceDeath();
            StartCoroutine(Dying());
        }

        public void SetGravity(float gravityScale = 1) {
            _rigidbody.gravityScale = gravityScale;
        }

        #endregion


        #region Protected methods
        #endregion


        #region Private methods
        public float line = 0.51f;

        private void FixedUpdate() {
            if (_rigidbody.gravityScale != 0) {
                _wasGrounded = _isGrounded;
                int layerMask = LayerMask.GetMask("Default");
                RaycastHit2D rayHit2 = Physics2D.Raycast((Vector2)transform.position /*+ Vector2.right * 0.1f*/, Vector2.down, 0.51f, layerMask);
                Debug.DrawLine(transform.position + Vector3.left * 0.1f, transform.position /*+ Vector3.left * 0.1f*/ + Vector3.down * line);
                if (/*rayHit1.transform ||*/ rayHit2.transform) {
                    if (!_wasGrounded) {
                        //Debug.Log("Play");
                        _jumpCount = 1;
                        _animatorAudioProxy.PlayLand();
                    }
                    _isGrounded = true;
                    if (!WaveMods.instance.CurrentWave.areObjectsSliding) {
                        float val = 1;
                        if (rayHit2.normal != Vector2.up) {
                            val *= Vector2.Angle(rayHit2.normal, Vector2.up) / 180f;
                        }
                        _rigidbody.velocity *= val;
                    }
                } else {
                    if (_wasGrounded) {
                        _jumpCount = 1;
                    }
                    _isGrounded = false;
                }
            } else {
                _wasGrounded = false;
            }
            if (_rigidbody.gravityScale == 0) {
                _rigidbody.AddForce(moveDirection * Time.deltaTime * speed * 10f);
            } else if (WaveMods.instance.CurrentWave.areObjectsSliding)
            {
                if (_isGrounded) {
                    _rigidbody.AddForce(moveDirection * Time.deltaTime * speed * 100f);
                } else {
                    transform.Translate(moveDirection * Time.deltaTime * speed);
                }
            } else {
                transform.Translate(moveDirection * Time.deltaTime * speed);
            }
        }

        

        private bool CheckDynamicObjectInRange(Pickupable pickupable) {
            //Debug.Log(Vector2.Angle(pickupable.transform.position - transform.position, _target.transform.position - transform.position));
            if (Vector2.Angle(pickupable.transform.position - transform.position, _target.transform.position - transform.position) > 45f) {
                return false;
            }
            RaycastHit2D targetRaycast = Physics2D.Raycast(transform.position, pickupable.transform.position - transform.position, 2f);
            if (targetRaycast.transform && targetRaycast.transform.gameObject.layer == LayerMask.NameToLayer("Pickupable")) {
                if (pickupable.owner != this) {
                    _pickedObject = pickupable;
                    pickupable.SetOwner(this);
                    return true;
                }
            }
            return false;
        }


        private void SetWaveMods()
        {
            _rigidbody.gravityScale = WaveMods.instance.CurrentWave.playerGravity;
        }


        private void FlipX(bool facingRight)
        {
            _facingRight = facingRight;
            _skeletonAnimator.skeleton.flipX = !_facingRight;
        }


        private IEnumerator Dying()
        {
            _isDead = true;
            if (Random.Range(0,2) == 0)
            {
                _animator.SetTrigger(die1Anim);
            }
            else
            {
                _animator.SetTrigger(die2Anim);
            }
            CreateBloodMark();
            var r = new RumbleEvent(playerIndex, 1f, 0.5f, Vector2.one);
            VibrationManager.AddRumble(r);
            yield return new WaitForSeconds(_respawnDelay);
            _animator.Rebind();
            transform.position = WaveMods.instance.CurrentWave.isLava ? _spawnPointHigh.position : _spawnPoint.position;
            _isDead = false;
        }


        private void RespawnOnGoal()
        {
            transform.position = WaveMods.instance.CurrentWave.isLava ? _spawnPointHigh.position : _spawnPoint.position;
        }


        private void CreateBloodMark()
        {
            int rnd = Random.Range(0, _bloodMarks.Length);
            var blood = Instantiate(_bloodMarks[rnd]);
            blood.transform.position = transform.position;
            blood.transform.position -= new Vector3(0f, 0.28f, 0f);
        }

        #endregion

    } //class

} //namespace
