﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using XInputDotNetPure;


namespace KrakJam
{
    public class VibrationManager
    {
        private static List<RumbleEvent> rumbleEvents = new List<RumbleEvent>();

        public static void AddRumble(RumbleEvent rumble)
        {
            rumbleEvents.Add(rumble);
        }


        public static void Clear()
        {
            rumbleEvents.Clear();
        }


        public static void Update()
        {
            // Ignore if there are no events
            if (rumbleEvents.Count > 0)
            {
                Vector2 currentPower = new Vector2(0,0);

                for (int i = 0; i < rumbleEvents.Count; ++i)
                {
                    // Update current event
                    rumbleEvents[i].Update();

                    if (rumbleEvents[i].timer > 0)
                    {
                        // Calculate current power
                        float timeLeft = Mathf.Clamp(rumbleEvents[i].timer / rumbleEvents[i].fadeTime, 0f, 1f);
                        currentPower = new Vector2(Mathf.Max(rumbleEvents[i].power.x * timeLeft, currentPower.x),
                                                   Mathf.Max(rumbleEvents[i].power.y * timeLeft, currentPower.y));

                        // Apply vibration to gamepad motors
                        GamePad.SetVibration(rumbleEvents[i].player, currentPower.x, currentPower.y);
                    }
                    else
                    {
                        // Remove expired event
                        rumbleEvents.Remove(rumbleEvents[i]);
                    }
                }
            }
        }

    } //class

} //namespace
